from MARC21 import *
import string
import fnmatch

class ReformedMARC21Record:
	#-- Transforms the interface to a MARC21 record.
	#-- Fields can be accessed using:
	#-- FIELD INDICATORS$SUBFIELD
	#-- ie, 650 22$a
	#-- Clients can read the contents of the fields
	#-- using shell-style patterns.
	#-- ie, 650 ??$a
	#-- Returns in a list the contents of all 650 tags
	#-- regardless of indicator.
	def __init__(self,M):
		self.rM={}
		tags=M.fields()
		for tag in tags:
			if type(M[tag])==type(''):
				id1=None
				id2=None
				subfield=None
				MARCutl=self.makeMARCutl(tag,id1,id2,subfield)
				if self.rM.has_key(MARCutl):
					self.rM[MARCutl].append(M[tag])
				else:
					self.rM[MARCutl]=[M[tag]]
			else:	
				if type(M[tag])==type([]):
					tagInstances=M[tag]
				else:
					tagInstances=[M[tag]]

				for tagInstance in tagInstances:
					if tagInstance.indicator1==' ':
						id1='#'
					else:
						id1=tagInstance.indicator1
					if tagInstance.indicator2==' ':
						id2='#'
					else:
						id2=tagInstance.indicator2

					subfields=tagInstance.subfields()
					for subfield in subfields:
						if type(tagInstance[subfield])==type([]):
							contents=tagInstance[subfield]
						else:
							contents=[tagInstance[subfield]]
						#-- Construct the MARC unique tag locator for this string.
						MARCutl=self.makeMARCutl(tag,id1,id2,subfield)
						if self.rM.has_key(MARCutl):
							self.rM[MARCutl]+=contents
						else:
							self.rM[MARCutl]=contents
		
	def makeMARCutl(self,tag,id1,id2,subfield):
		utl=tag+' '
		if (id1 and id2 and subfield):
			utl+='%s%s'%(id1,id2)
			utl+='$'+subfield
		return utl

	def isFNPattern(self,tagPattern):
		#-- The tagPattern should not
		#-- contain any ?,*,or []'s if it is a literal.
		qmark='?' in tagPattern
		amark='*' in tagPattern
		bmark=('[' in tagPattern) or (']' in tagPattern)
		if qmark or amark or bmark:
			return 1
		else:
			return 0

	def __getitem__(self,tagPattern):
		contents=[]
		#-- A sort of optimization.
		#-- Check to see if the tagPattern is a literal
		#-- or a pattern.
		if self.isFNPattern(tagPattern):
			tags=self.rM.keys()
			for tag in tags:
				if fnmatch.fnmatch(tag,tagPattern):
					contents+=self.rM[tag]
		else:
			if self.rM.has_key(tagPattern):
				contents=self.rM[tagPattern][:]
		return contents

def stripTrailingPuncs(s):
	removechars=string.punctuation+string.whitespace
	i=len(s)-1
	while ((i>0) and (s[i] in removechars)):
		i=i-1
	return s[:i+1]

class MARCxUDC:
	def __init__(self,map):
		if type(map)==type({}):
			self.dcMap=map
		else:
			self.dcMap={}
			self.parseMapFile(map)

	def parseMapFile(self, map):
		f=open(map)
		l=f.readline()
		while l!="":
			l=string.strip(l)
			if len(l):
				#-- Is this a comment line?
				if l[0]=='#':
					pass
				else:
					i=string.find(l,'=')
					dcElement=string.strip(l[:i])
					MARCElement=string.strip(l[i+1:])
					if self.dcMap.has_key(dcElement):
						self.dcMap[dcElement].append(MARCElement)
					else:
						self.dcMap[dcElement]=[MARCElement]
			l=f.readline()
		f.close()

	def makeDCContainer(self):
		dc={}
		for dcElement in self.dcMap.keys():
			dc[dcElement]=[]
		return dc

	def map(self,M):
		rM=ReformedMARC21Record(M)
		dc=self.makeDCContainer()
		for dcElement in self.dcMap.keys():
			for MARCtag in self.dcMap[dcElement]:
				dc[dcElement]+=rM[MARCtag]
		return dc

def toXML(mapped):
	texts=[]
	texts.append('<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dc="http://purl.org/dc/elements/1.0">\n')
	texts.append('\t<rdf:Description>\n')
	for element in mapped.keys():
		for instance in mapped[element]:
			#cleanedInstance=stripTrailingPuncs(instance)
			cleanedInstance=instance
			texts.append('\t\t<dc:%s>%s<dc:/%s>\n'%(element,cleanedInstance,element))
	texts.append('\t</rdf:Description>\n')
	texts.append('</rdf:RDF>\n')
	text=''.join(texts)
	return text

if __name__=='__main__':
	import sys

	mapFile=sys.argv[1]
	inFile=sys.argv[2]
	outFile=sys.argv[3]

	Rs=[]
	F=MARC21File(inFile)
	R=F.next()
	while R:
		Rs.append(R)
		R=F.next()

	import time
	begin_time=time.time()

	Map=MARCxUDC(mapFile)
	f=open(outFile,'wt')
	for R in Rs:
		f.write(toXML(Map.map(R)))
		f.write('\n')
	f.close()

	end_time=time.time()
	print "Processed %s records in %s seconds"%(len(Rs),end_time-begin_time)

